-- ***************************************************************************
-- *  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de)
-- *  All rights reserved.
-- *
-- *  Redistribution and use in source and binary forms, with or without 
-- *  modification, are permitted provided that the following conditions 
-- *  are met:
-- *  
-- *  1. Redistributions of source code must retain the above copyright 
-- *     notice, this list of conditions and the following disclaimer.
-- *  2. Redistributions in binary form must reproduce the above copyright
-- *     notice, this list of conditions and the following disclaimer in the 
-- *     documentation and/or other materials provided with the distribution.
-- *  3. Neither the name of the author nor the names of its contributors may 
-- *     be used to endorse or promote products derived from this software 
-- *     without specific prior written permission.
-- *
-- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
-- *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
-- *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
-- *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
-- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
-- *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
-- *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
-- *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
-- *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
-- *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
-- *  SUCH DAMAGE.
-- *
-- ***************************************************************************
-- *  History:
-- *
-- *  22.08.2017  mifi  First version.
-- ***************************************************************************

-- ***************************************************************************
-- *  DEFINE: Library                                                        *
-- ***************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


-- ***************************************************************************
-- *  DEFINE: Entity                                                         *
-- ***************************************************************************

entity cpu2av is
   port (  
      clk_i            : in  std_logic                      := '0';
      rst_i            : in  std_logic                      := '0';

      -- CPU interface
      cpu_address_i    : in  std_logic_vector(4 downto 0)   := (others => '0');
      cpu_read_i       : in  std_logic                      := '0';
      cpu_readdata_o   : out std_logic_vector(7 downto 0);
      cpu_write_i      : in  std_logic                      := '0';
      cpu_writedata_i  : in  std_logic_vector(7 downto 0)   := (others => '0');

      -- Avalon Slave interface      
      avs_address_i    : in  std_logic_vector(2 downto 0)   := (others => '0');
      avs_read_i       : in  std_logic                      := '0';
      avs_readdata_o   : out std_logic_vector(31 downto 0);
      avs_write_i      : in  std_logic                      := '0';
      avs_writedata_i  : in  std_logic_vector(31 downto 0)  := (others => '0')
   );
end entity cpu2av;


-- ***************************************************************************
-- *  DEFINE: Architecture                                                   *
-- ***************************************************************************

architecture syn of cpu2av is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------
   
   constant ID5            : std_logic_vector(7 downto 0)  := x"42";  -- 'B'
   constant ID6            : std_logic_vector(7 downto 0)  := x"4F";  -- 'O'
   
   -- CPU interface
   constant CPU_ADR_DATA   : std_logic_vector(4 downto 0) := "00" & "000";   -- Read / Write
   constant CPU_ADR_FRXC   : std_logic_vector(4 downto 0) := "00" & "001";   -- Read / Write
   constant CPU_ADR_FTXC   : std_logic_vector(4 downto 0) := "00" & "010";   -- Read / Write
   constant CPU_ADR_LSR    : std_logic_vector(4 downto 0) := "00" & "011";   -- Read only
   constant CPU_ADR_RES_4  : std_logic_vector(4 downto 0) := "00" & "100";   -- Reserve 1
   constant CPU_ADR_ID5    : std_logic_vector(4 downto 0) := "00" & "101";   -- Read only
   constant CPU_ADR_ID6    : std_logic_vector(4 downto 0) := "00" & "110";   -- Read only
   constant CPU_ADR_INFO   : std_logic_vector(4 downto 0) := "00" & "111";   -- Read only
   
   -- Avalon Slave interface   
   constant AVS_ADR_DATA   : std_logic_vector(2 downto 0)  := "000"; -- Read / Write
   constant AVS_ADR_FRXC   : std_logic_vector(2 downto 0)  := "001"; -- Read / Write
   constant AVS_ADR_FTXC   : std_logic_vector(2 downto 0)  := "010"; -- Read / Write
   constant AVS_ADR_LSR    : std_logic_vector(2 downto 0)  := "011"; -- Read
   constant AVS_ADR_RES_4  : std_logic_vector(2 downto 0)  := "100"; -- Reserve 4
   constant AVS_ADR_RES_5  : std_logic_vector(2 downto 0)  := "101"; -- Reserve 5
   constant AVS_ADR_RES_6  : std_logic_vector(2 downto 0)  := "110"; -- Reserve 6
   constant AVS_ADR_INFO   : std_logic_vector(2 downto 0)  := "111"; -- Read / Write

   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------
   
   component fifo
      port (
         clock  : in std_logic ;
         data   : in std_logic_vector (7 downto 0);
         rdreq  : in std_logic ;
         sclr   : in std_logic ;
         wrreq  : in std_logic ;
         empty  : out std_logic ;
         full   : out std_logic ;
         q      : out std_logic_vector (7 downto 0);
         usedw  : out std_logic_vector (7 downto 0)
      );
   end component;
   
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------
   
   -- CPU
   signal cpu_readdata     : std_logic_vector(7 downto 0) := (others => '0');
   signal cpu_read_mux     : std_logic_vector(7 downto 0) := (others => '0');
   
   signal cpu_info         : std_logic_vector(7 downto 0) := (others => '0'); 
   signal cpu_lsr          : std_logic_vector(7 downto 0) := (others => '0'); 
   signal cpu_lsr_thre     : std_logic := '0';
   signal cpu_lsr_dr       : std_logic := '0';
   signal cpu_tx_fifo_clr  : std_logic := '1'; 
   signal cpu_rx_fifo_clr  : std_logic := '1'; 
   
   -- TX-FIFO (CPU to Avalon)
   signal tx_fifo_data     : std_logic_vector (7 downto 0) := (others => '0');
   signal tx_fifo_wrreq    : std_logic := '0';
   signal tx_fifo_empty    : std_logic := '0';
   signal tx_fifo_full     : std_logic := '0';
   signal tx_fifo_usedw    : std_logic_vector (7 downto 0) := (others => '0');
   
   signal rx2av_rdreq      : std_logic := '0';
   signal rx2av_data       : std_logic_vector (7 downto 0) := (others => '0');
   
   signal rx2av_rdreq_sr   : std_logic_vector(1 downto 0) := (others => '0');
   signal tx_fifo_wrreq_sr : std_logic_vector(1 downto 0) := (others => '0');
   
   -- Avalon
   signal av_readdata      : std_logic_vector(7 downto 0) := (others => '0');
   signal av_read_mux      : std_logic_vector(7 downto 0) := (others => '0');

   signal av_lsr           : std_logic_vector(7 downto 0) := (others => '0'); 
   signal av_lsr_thre      : std_logic := '0';
   signal av_lsr_dr        : std_logic := '0';
   signal av_tx_fifo_clr   : std_logic := '1'; 
   signal av_rx_fifo_clr   : std_logic := '1'; 
   signal av_info          : std_logic_vector(7 downto 0) := (others => '0'); 
   signal av_info_ready    : std_logic := '0';
   
   -- Avalon to CPU FIFO
   signal rx_fifo_data     : std_logic_vector (7 downto 0) := (others => '0');
   signal rx_fifo_wrreq    : std_logic := '0';
   signal rx_fifo_empty    : std_logic := '0';
   signal rx_fifo_full     : std_logic := '0';
   signal rx_fifo_usedw    : std_logic_vector (7 downto 0) := (others => '0');

   signal rx2cpu_rdreq     : std_logic := '0';
   signal rx2cpu_data      : std_logic_vector (7 downto 0) := (others => '0');
   
   signal rx2cpu_rdreq_sr  : std_logic_vector(1 downto 0) := (others => '0');
   signal rx_fifo_wrreq_sr : std_logic_vector(1 downto 0) := (others => '0');
   
begin

   --------------------------------------------------------
   -- Host Interface
   --------------------------------------------------------

   --   
   -- Host Register File
   --
   process (clk_i, rst_i)
   begin
      if (rst_i = '1') then
         cpu_tx_fifo_clr <= '1';
         cpu_rx_fifo_clr <= '1';

      elsif rising_edge(clk_i) then
      
         -- Default
         cpu_tx_fifo_clr <= '0';
         cpu_rx_fifo_clr <= '0';
      
         if (cpu_write_i = '1') then
         
            -- Address decoder
            case cpu_address_i is
            
               when CPU_ADR_DATA =>
                  tx_fifo_data <= cpu_writedata_i(7 downto 0);
                  
               when CPU_ADR_FRXC => 
                  cpu_rx_fifo_clr <= '1';

               when CPU_ADR_FTXC => 
                  cpu_tx_fifo_clr <= '1';
            
               when others =>
                  -- Do nothing
            
            end case;
         end if;
         
      end if; 
   end process;
   
   --
   -- Handle tx_fifo_wrreq and rx2cpu_rdreq
   --
   process (clk_i, rst_i)
   begin
      if (rst_i = '1') then
         tx_fifo_wrreq    <= '0';
         rx2cpu_rdreq     <= '0';
         tx_fifo_wrreq_sr <= (others => '0');
         rx2cpu_rdreq_sr  <= (others => '0');
      
      elsif rising_edge(clk_i) then
      
         -- Default
         tx_fifo_wrreq <= '0';
         rx2cpu_rdreq  <= '0';


         if ((cpu_write_i = '1') AND (CPU_ADR_DATA = cpu_address_i)) then
            tx_fifo_wrreq_sr <= tx_fifo_wrreq_sr(0) & '1';
         else   
            tx_fifo_wrreq_sr <= tx_fifo_wrreq_sr(0) & '0';
         end if;

         if ((cpu_read_i = '1') AND (CPU_ADR_DATA = cpu_address_i)) then
            rx2cpu_rdreq_sr <= rx2cpu_rdreq_sr(0) & '1';
         else   
            rx2cpu_rdreq_sr <= rx2cpu_rdreq_sr(0) & '0';
         end if;
         
         
         -- Check for falling edge
         if ((tx_fifo_wrreq_sr(1) = '1') AND (tx_fifo_wrreq_sr(0) = '0')) then
            tx_fifo_wrreq <= '1';
         end if;
         
         if ((rx2cpu_rdreq_sr(1) = '1') AND (rx2cpu_rdreq_sr(0) = '0')) then
            rx2cpu_rdreq <= '1';
         end if;
         
      end if;
   end process;
   
   --
   -- CPU 2 Avalon FIFO
   -- From the CPU point of view is this the TX-FIFO
   -- and from the Avalon point of view is this the RX-FIFO
   --
   inst_tx_fifo : fifo
      port map (
         clock  => clk_i,
         data   => tx_fifo_data,      
         rdreq  => rx2av_rdreq,
         sclr   => cpu_tx_fifo_clr OR av_rx_fifo_clr,
         wrreq  => tx_fifo_wrreq,
         empty  => tx_fifo_empty,
         full   => tx_fifo_full,
         q      => rx2av_data,
         usedw  => tx_fifo_usedw
      );

   cpu_lsr_thre <= NOT tx_fifo_full;
   av_lsr_dr    <= NOT tx_fifo_empty;
   
   cpu_lsr <= "00"         &  -- Bit 7..6
              cpu_lsr_thre &  -- Bit 5
              "0000"       &  -- Bit 4..1
              cpu_lsr_dr;     -- Bit 0
              
   --
   -- Host Read Multiplexer
   --
   cpu_read_mux <= rx2cpu_data   when (CPU_ADR_DATA = cpu_address_i) else 
                   rx_fifo_usedw when (CPU_ADR_FRXC = cpu_address_i) else 
                   tx_fifo_usedw when (CPU_ADR_FTXC = cpu_address_i) else 
                   cpu_lsr       when (CPU_ADR_LSR  = cpu_address_i) else 
                   ID5           when (CPU_ADR_ID5  = cpu_address_i) else 
                   ID6           when (CPU_ADR_ID6  = cpu_address_i) else
                   cpu_info      when (CPU_ADR_INFO = cpu_address_i) else (others => '0'); 
                   

   process (clk_i)
   begin
      if rising_edge(clk_i) then
         if (cpu_read_i = '1') then
            cpu_readdata <= cpu_read_mux;
         end if;
      end if;
   end process;      
   
   
   --------------------------------------------------------
   -- Avalon Interface
   --------------------------------------------------------

   --   
   -- Avalon Register File
   --
   process (clk_i, rst_i)
   begin
      if (rst_i = '1') then
         av_tx_fifo_clr <= '1';        
         av_rx_fifo_clr <= '1';        
         av_info        <= (others => '0');

      elsif rising_edge(clk_i) then
      
         -- Default
         av_tx_fifo_clr <= '0';        
         av_rx_fifo_clr <= '0';        
      
         if (avs_write_i = '1') then
         
            -- Address decoder
            case avs_address_i is
            
               when AVS_ADR_DATA =>
                  rx_fifo_data <= avs_writedata_i(7 downto 0);
                  
               when AVS_ADR_FRXC => 
                  av_rx_fifo_clr <= '1';
                  
               when AVS_ADR_FTXC => 
                  av_tx_fifo_clr <= '1';

               when AVS_ADR_INFO => 
                  av_info  <= avs_writedata_i(7 downto 0);
                  cpu_info <= avs_writedata_i(7 downto 0);
            
               when others =>
                  -- Do nothing
            
            end case;
         end if;
         
      end if; 
   end process;
   
   --
   -- Avalon CTRL register
   --
   av_info_ready <= av_info(7);

   --
   -- Handle rx_fifo_wrreq and rx2av_rdreq
   --
   process (clk_i, rst_i)
   begin
      if (rst_i = '1') then
         rx_fifo_wrreq    <= '0';
         rx2av_rdreq      <= '0';
         rx_fifo_wrreq_sr <= (others => '0');
         rx2av_rdreq_sr   <= (others => '0');
      
      elsif rising_edge(clk_i) then
      
         -- Default
         rx_fifo_wrreq <= '0';
         rx2av_rdreq   <= '0';


         if ((avs_write_i = '1') AND (AVS_ADR_DATA = avs_address_i)) then
            rx_fifo_wrreq_sr <= rx_fifo_wrreq_sr(0) & '1';
         else   
            rx_fifo_wrreq_sr <= rx_fifo_wrreq_sr(0) & '0';
         end if;

         if ((avs_read_i = '1') AND (AVS_ADR_DATA = avs_address_i)) then
            rx2av_rdreq_sr <= rx2av_rdreq_sr(0) & '1';
         else   
            rx2av_rdreq_sr <= rx2av_rdreq_sr(0) & '0';
         end if;
         
         
         -- Check for falling edge
         if ((rx_fifo_wrreq_sr(1) = '1') AND (rx_fifo_wrreq_sr(0) = '0')) then
            rx_fifo_wrreq <= '1';
         end if;
         
         if ((rx2av_rdreq_sr(1) = '1') AND (rx2av_rdreq_sr(0) = '0')) then
            rx2av_rdreq <= '1';
         end if;
         
         
      end if;
   end process;
   
   --
   -- Avalon 2 CPU FIFO
   -- From the CPU point of view is this the RX-FIFO
   -- and from the Avalon point of view is this the TX-FIFO
   --
   inst_rx_fifo : fifo
      port map (
         clock  => clk_i,
         data   => rx_fifo_data,
         rdreq  => rx2cpu_rdreq,
         sclr   => cpu_rx_fifo_clr OR av_tx_fifo_clr,
         wrreq  => rx_fifo_wrreq,
         empty  => rx_fifo_empty,
         full   => rx_fifo_full, 
         q      => rx2cpu_data,
         usedw  => rx_fifo_usedw
      );

   av_lsr_thre <= NOT rx_fifo_full;
   cpu_lsr_dr  <= NOT rx_fifo_empty;
   
   av_lsr <= "00"         &   -- Bit 7..6
              av_lsr_thre &   -- Bit 5
              "0000"      &   -- Bit 4..1
              av_lsr_dr;      -- Bit 0
              
   --
   -- Avalon Read Multiplexer
   --
   av_read_mux <= rx2av_data    when (AVS_ADR_DATA = avs_address_i) else 
                  tx_fifo_usedw when (AVS_ADR_FRXC = avs_address_i) else 
                  rx_fifo_usedw when (AVS_ADR_FTXC = avs_address_i) else 
                  av_lsr        when (AVS_ADR_LSR  = avs_address_i) else
                  av_info       when (AVS_ADR_INFO = avs_address_i) else (others => '0'); 

   process (clk_i)
   begin
      if rising_edge(clk_i) then
         if (avs_read_i = '1') then
            av_readdata <= av_read_mux;
         end if;
      end if;
   end process;      
              
              
   --------------------------------------------------------
   -- Output
   --------------------------------------------------------

   -- CPU interface
   cpu_readdata_o <= cpu_readdata;
   
   -- Avalon Slave interface
   avs_readdata_o <= x"000000" & av_readdata;  
   
end architecture syn;

-- *** EOF ***

