-- ***************************************************************************
-- *  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de)
-- *  All rights reserved.
-- *
-- *  Redistribution and use in source and binary forms, with or without 
-- *  modification, are permitted provided that the following conditions 
-- *  are met:
-- *  
-- *  1. Redistributions of source code must retain the above copyright 
-- *     notice, this list of conditions and the following disclaimer.
-- *  2. Redistributions in binary form must reproduce the above copyright
-- *     notice, this list of conditions and the following disclaimer in the 
-- *     documentation and/or other materials provided with the distribution.
-- *  3. Neither the name of the author nor the names of its contributors may 
-- *     be used to endorse or promote products derived from this software 
-- *     without specific prior written permission.
-- *
-- *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
-- *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
-- *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
-- *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
-- *  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
-- *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
-- *  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
-- *  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
-- *  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
-- *  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
-- *  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
-- *  SUCH DAMAGE.
-- *
-- ***************************************************************************
-- *  History:
-- *
-- *  30.08.2017  mifi  First version.
-- ***************************************************************************

-- ***************************************************************************
-- *  DEFINE: Library                                                        *
-- ***************************************************************************
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;


-- ***************************************************************************
-- *  DEFINE: Entity                                                         *
-- ***************************************************************************

--
-- CPU_XX are the original not synchronized bus signals. 
-- The synchronisation will be done by the module here.
--
entity cpusync is
   port (  
      clk_i            : in    std_logic := '0';
      rst_i            : in    std_logic := '0';

      -- CPU interface
      CPU_A            : in    std_logic_vector(4 downto 0) := (others => '0');
      CPU_D            : inout std_logic_vector(7 downto 0);
      CPU_CS_n         : in    std_logic := '1';
      CPU_RD_n         : in    std_logic := '1';
      CPU_WR_n         : in    std_logic := '1';

      -- CPU sync interface
      cpu_address_o    : out   std_logic_vector(4 downto 0);
      cpu_read_o       : out   std_logic;
      cpu_readdata_i   : in    std_logic_vector(7 downto 0) := (others => '0');
      cpu_write_o      : out   std_logic;
      cpu_writedata_o  : out   std_logic_vector(7 downto 0)
   );
end entity cpusync;


-- ***************************************************************************
-- *  DEFINE: Architecture                                                   *
-- ***************************************************************************

architecture syn of cpusync is

   --------------------------------------------------------
   -- Define all constants here
   --------------------------------------------------------

   --------------------------------------------------------
   -- Define all components which are included here
   --------------------------------------------------------
   
   --------------------------------------------------------
   -- Define all local signals here
   --------------------------------------------------------
   
   signal cs_sr      : std_logic_vector(2 downto 0) := (others => '0');
   signal rd_sr      : std_logic_vector(2 downto 0) := (others => '0');
   signal wr_sr      : std_logic_vector(2 downto 0) := (others => '0');
   signal cpu_read   : std_logic := '0';
   signal cpu_write  : std_logic := '0';
   
begin

   --
   -- Synchronize CPU control signals
   --
   process (clk_i, rst_i)
   begin
   
      if (rst_i = '1') then
         cs_sr <= (others => '0');
         rd_sr <= (others => '0');
         wr_sr <= (others => '0');
         
      elsif rising_edge(clk_i) then
         cs_sr <= cs_sr(1 downto 0) & NOT CPU_CS_n;
         rd_sr <= rd_sr(1 downto 0) & NOT CPU_RD_n;
         wr_sr <= wr_sr(1 downto 0) & NOT CPU_WR_n;
         
      end if; 
      
   end process;
   

   --
   -- Create cpu_read and cpu_write
   --
   process(clk_i, rst_i)
   begin
      if (rst_i = '1') then
         cpu_read  <= '0';
         cpu_write <= '0';
   
      elsif rising_edge(clk_i) then
      
         -- Default
         cpu_read  <= '0';
         cpu_write <= '0';
         
         if (cs_sr(1) = '1') then
         
            -- Check for rising edge
            if ((rd_sr(2) = '0') AND (rd_sr(1) = '1')) then
               cpu_read <= '1';
            end if;
            
            if ((wr_sr(2) = '0') AND (wr_sr(1) = '1')) then
               cpu_write <= '1';
            end if;
            
         end if;
      
      end if; 
   end process;
   
   
   --------------------------------------------------------
   -- Output
   --------------------------------------------------------
   
   -- Control
   cpu_read_o      <= cpu_read;
   cpu_write_o     <= cpu_write;
   
   -- Address 
   cpu_address_o   <= CPU_A;
   
   -- Data
   CPU_D           <= cpu_readdata_i(7 downto 0) when ((CPU_CS_n = '0') AND (CPU_RD_n = '0')) else (others => 'Z');
   cpu_writedata_o <= CPU_D;
	
end architecture syn;

-- *** EOF ***

