/**************************************************************************
*  Copyright (c) 2017 by Michael Fischer (www.emb4fun.de)
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  28.08.2017  mifi  First Version, based on the "COM" version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include "project.h"
#include "adler32.h"
#include "xboot.h"
#include "rsu.h"

#include "system.h"
#include "sys/alt_irq.h"
#include "sys/alt_cache.h"
#include "sys/alt_stdio.h"
#include "sys/ioctl.h"
#include "altera_avalon_timer_regs.h"
#include "altera_avalon_uart.h"
#include "epcs_commands.h"

/*=======================================================================*/
/*  Extern                                                               */
/*=======================================================================*/

int altera_avalon_uart_read (altera_avalon_uart_state* sp, char* ptr, int len, int flags);

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * COM
 */
typedef struct
{
  volatile uint32_t DATA;
  volatile uint32_t FRXC;
  volatile uint32_t FTXC;
  volatile uint32_t LSR;
  volatile uint32_t Reserve4;
  volatile uint32_t Reserve5;
  volatile uint32_t Reserve6;
  volatile uint32_t INFO;
} UART_TypeDef;

#define UART_FIFO_SIZE  256

#define LSR_DR          (1<<0)
#define LSR_THRE        (1<<5)

 
/*
 * Terminal
 */
#if !defined(PRINTF_BUFFER_SIZE)
#define PRINTF_BUFFER_SIZE    256
#endif


/*
 * Flash
 */
#define FLASH_USER_IMG_SECTOR_COUNT ((FLASH_END_ADDR - FLASH_IMG_START_ADDR) / FLASH_SECTOR_SIZE)


/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*
 * System
 */
static volatile uint32_t dSystemTick = 0;

/*
 * COM
 */
static UART_TypeDef* UART = (UART_TypeDef*)(AVS2CON32_BASE);
static uint8_t      bInfo = 0;

/*
 * Terminal
 */
static uint8_t PrintfBuffer[PRINTF_BUFFER_SIZE];

static altera_avalon_uart_state UARTState;


/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*************************************************************************/
/*  SysTickHandler                                                       */
/*                                                                       */
/*  In    : context                                                      */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void SysTickHandler (void* context)
{
   (void)context;
   
   /* Clear the interrupt */
   IOWR_ALTERA_AVALON_TIMER_STATUS(SYSTICK_BASE, 0);
   
   /* 
    * Dummy read to ensure IRQ is negated before the ISR returns.
    * The control register is read because reading the status
    * register has side-effects per the register map documentation.
    */
   IORD_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE);

   /* Handle SysTick counter */   
   dSystemTick++;
    
} /* SysTickHandler */

/*************************************************************************/
/*  InitSysTick                                                          */
/*                                                                       */
/*  Init the SysTick timer.                                              */
/*                                                                       */                                                                       
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void InitSysTick (void)
{
   alt_irq_context   status;
   static uint8_t   bInitDone = 0;
   uint32_t         dHirResPeriod;
   uint16_t         wValue;
   
   if (0 == bInitDone)
   {
      bInitDone = 1;

      /* Calculate timer period value */
      dHirResPeriod = (SYSTICK_FREQ / TICKS_PER_SECOND) - 1;
      
      /* Set period register */
      IOWR_ALTERA_AVALON_TIMER_PERIODH(SYSTICK_BASE, ((dHirResPeriod>>16) & 0xFFFF));      
      IOWR_ALTERA_AVALON_TIMER_PERIODL(SYSTICK_BASE, (dHirResPeriod & 0xFFFF));

      /* Set control register, stop timer */      
      wValue = ALTERA_AVALON_TIMER_CONTROL_STOP_MSK |
               ALTERA_AVALON_TIMER_CONTROL_CONT_MSK |
               ALTERA_AVALON_TIMER_CONTROL_ITO_MSK;
      IOWR_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE, wValue);
      
      /* Clear status */
      IOWR_ALTERA_AVALON_TIMER_STATUS(SYSTICK_BASE, 0);
  
      /* Set SysTick interrupt handler */
      status = alt_irq_disable_all();  
      alt_ic_isr_register(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ, SysTickHandler, 0x0, 0x0);
      alt_ic_irq_disable(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ);
      alt_irq_enable_all(status);

      /* Set control register, start timer */      
      wValue = ALTERA_AVALON_TIMER_CONTROL_START_MSK |
               ALTERA_AVALON_TIMER_CONTROL_CONT_MSK  |
               ALTERA_AVALON_TIMER_CONTROL_ITO_MSK;
      IOWR_ALTERA_AVALON_TIMER_CONTROL(SYSTICK_BASE, wValue);

      /* Enable SysTick interrupt */   
      alt_ic_irq_enable(SYSTICK_IRQ_INTERRUPT_CONTROLLER_ID, SYSTICK_IRQ);
   }      
   
} /* InitSysTick */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  OSTimeDly                                                            */
/*                                                                       */
/*  Loop for a specified number of milliseconds,                         */
/*  will not release the CPU.                                            */
/*                                                                       */
/*  In    : TimeoutMs                                                    */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void OSTimeDly (uint32_t dTimeoutMs)
{
   uint32_t dTicks;   
   uint32_t dTimeEnd;
   
   dTicks   = OS_MS_2_TICKS(dTimeoutMs);
   dTimeEnd = dSystemTick + dTicks;
   
   while (dSystemTick <= dTimeEnd)
   {
      __asm__ volatile ("nop");
   }
      
} /* OSTimeDly */

/*************************************************************************/
/*  OSTimeGet                                                            */
/*                                                                       */
/*  Return the time from the system ticker.                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Systick                                                      */
/*************************************************************************/
uint32_t OSTimeGet (void)
{
   return(dSystemTick);
} /* OSTimeGet */

/*************************************************************************/
/*  SystemInit                                                           */
/*                                                                       */
/*  Initialize the system.                                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void SystemInit (void)
{
   /* Init SysTick */
   InitSysTick();
   
   /* Set Avalon UART base */
   UARTState.base = UART_BASE;
   
   /* Initialize the UART, disable cache if available */
   UART = (UART_TypeDef*)((uint32_t)UART | 0x80000000);
   
   /* Reset RX and TX_FIFO */
   UART->FRXC = 0;
   UART->FTXC = 0;
   
   UART->INFO = bInfo;
   
} /* SystemInit */

/*************************************************************************/
/*  COMSendBlock                                                         */
/*                                                                       */
/*  Send a block of data.                                                */
/*                                                                       */
/*  In    : pData, wSize                                                 */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void COMSendBlock (uint8_t *pData, uint16_t wSize)
{
   uint32_t dLSR;

   while (wSize > 0)
   {
      dLSR = UART->LSR;
      if (dLSR & LSR_THRE)
      {
         UART->DATA = *pData++;
         wSize--;
      }   
   }      

} /* COMSendBlock */

/*************************************************************************/
/*  COMSendChar                                                          */
/*                                                                       */
/*  Send one character.                                                  */
/*                                                                       */
/*  In    : bData                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void COMSendChar (uint8_t bData)
{
   uint32_t dLSR;

   dLSR = UART->LSR;
   if (dLSR & LSR_THRE)
   {
      UART->DATA = bData;
   }   

} /* COMSendChar */

/*************************************************************************/
/*  COMGetChar                                                           */
/*                                                                       */
/*  Get one character if available.                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: EOF / character                                              */
/*************************************************************************/
int COMGetChar (void)
{
   uint32_t dLSR;
   int      nData = EOF;
   
   dLSR = UART->LSR;
   if (dLSR & LSR_DR)
   {
      nData = (int)UART->DATA;
   }   

   return(nData);
} /* COMGetChar */

/*************************************************************************/
/*  COMInfoMaskSet                                                       */
/*                                                                       */
/*  Set a info mask.                                                     */
/*                                                                       */
/*  In    : bMask                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void COMInfoMaskSet (uint8_t bMask)
{
   bInfo     |= bMask;
   UART->INFO = bInfo;

} /* COMInfoMaskSet */

/*************************************************************************/
/*  COMInfoMaskClr                                                       */
/*                                                                       */
/*  Clear a info mask.                                                   */
/*                                                                       */
/*  In    : bMask                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void COMInfoMaskClr (uint8_t bMask)
{
   bInfo     &= ~bMask;
   UART->INFO = bInfo;

} /* COMInfoMaskClr */

/*************************************************************************/
/*  term_Task                                                            */
/*                                                                       */
/*  This is the terminal task.                                           */
/*                                                                       */
/*  In    : dActualTime                                                  */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void term_Task (uint32_t dActualTime)
{
   uint8_t bData;
   int     nCount;
   
   (void)dActualTime;

   nCount = altera_avalon_uart_read(&UARTState, (char*)&bData, 1, O_NONBLOCK);
   if (1 == nCount)
   {
      term_RxCallback(bData);
   }
   
} /* term_Task */

/*************************************************************************/
/*  term_printf                                                          */
/*                                                                       */
/*  This is a printf like output function.                               */
/*  The function is based on a CrossWorks for ARM example.               */
/*  Use "Customizing putchar" in the help function of CrossWorks.        */
/*                                                                       */
/*  In    : fmt                                                          */
/*  Out   : none                                                         */
/*  Return: Number of characters transmitted                             */
/*************************************************************************/
int term_printf (const char *fmt, ...)
{
   int     n = 0; /* Number of characters transmitted */
   va_list ap;

   va_start(ap, fmt);
   n = vsnprintf((char*)PrintfBuffer, sizeof(PrintfBuffer), fmt, ap);
   va_end(ap);
   (void)ap;
  
   alt_putstr((const char*)PrintfBuffer);
  
   return(n);
} /* term_printf */

/*************************************************************************/
/*  term_puts                                                            */
/*                                                                       */
/*  Send a string over the UART.                                         */
/*                                                                       */
/*  In    : string                                                       */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void term_puts (char *string)
{
   alt_putstr((const char*)string);
   COMSendChar('\r');
   COMSendChar('\n');
   
} /* term_puts */

/*************************************************************************/
/*  term_putchar                                                         */
/*                                                                       */
/*  Send a character over the UART.                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: Return the char which was written or EOF                     */
/*************************************************************************/
int term_putchar (int ch)
{
   alt_putchar(ch);
   
   return(ch);
} /* term_putchar */

/*************************************************************************/
/*  FlashIsImageAvailable                                                */
/*                                                                       */
/*  Check if a image is available.                                       */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: 1 = OK / 0 = Error                                           */
/*************************************************************************/
int FlashIsImageAvailable (void)
{
   int         nRes = 0;
   uint32_t    dBaseAddress = EPCS_FLASH_CONTROLLER_BASE + EPCS_FLASH_CONTROLLER_REGISTER_OFFSET;
   uint32_t    dCRC; 
   XBOOT_HEADER Header;
   
   /* Read XBOOT header */
   epcs_read_buffer(dBaseAddress, FLASH_IMG_START_ADDR, (uint8_t*)&Header, sizeof(Header));
   
   /* Check header */
   if ((XBOOT_HEADER_MAGIC_1 == Header.dMagic1) &&
       (XBOOT_HEADER_MAGIC_2 == Header.dMagic2) &&
       (XBOOT_HEADER_SIZEVER == Header.dSizeVersion))
   {
      /* Check CRC of the header */
      dCRC = adler32(ADLER_START_VALUE, (const uint8_t *)&Header, sizeof(XBOOT_HEADER) - XBOOT_SIZE_OF_CRC32);
      if (dCRC == Header.dHeaderCRC32)
      {
         /* Valid CRC */
         nRes = 1;
      }
   }       
   
   return(nRes);
} /* FlashIsImageAvailable */

/*************************************************************************/
/*  FlashStartImage                                                      */
/*                                                                       */
/*  Start the flash image.                                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void FlashStartImage (void)
{
   uint32_t dBootAddress = FLASH_IMG_START_ADDR + sizeof(XBOOT_HEADER);
   
   term_printf("Start image at 0x%08X\r\n", (unsigned int)dBootAddress);
   OSTimeDly(50);
   
   /*
    * 
    * In this example we only need to ensure the state of the Nios II cpu is 
    * equivalent to reset.  If we disable interrupts, and flush the caches, 
    * then the program we jump to should receive the cpu just as it would 
    * coming out of a hardware reset.
    */
   alt_irq_disable_all();
   alt_dcache_flush_all();
   alt_icache_flush_all();
   
   rsu_factory_trigger_reconfig(REMOTE_UPDATE_CONTROLLER_BASE,
                                (dBootAddress >> 2),
                                0);

} /* FlashStartImage */

/*************************************************************************/
/*  FlashEraseAll                                                        */
/*                                                                       */
/*  Erase the flash user area.                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void FlashEraseAll (void)
{
   uint32_t dBaseAddress = EPCS_FLASH_CONTROLLER_BASE + EPCS_FLASH_CONTROLLER_REGISTER_OFFSET;
   uint32_t dAddress;
   uint16_t wCount;
   
   term_printf("  Flash erase 1 of %d\r", FLASH_USER_IMG_SECTOR_COUNT);
   for (wCount = 0; wCount < FLASH_USER_IMG_SECTOR_COUNT; wCount++)
   {
      dAddress = FLASH_IMG_START_ADDR + (wCount * FLASH_SECTOR_SIZE);
   
      epcs_write_enable(dBaseAddress);
      epcs_sector_erase(dBaseAddress, dAddress);
      term_printf("  Flash erase %d of %d\r", wCount+1, FLASH_USER_IMG_SECTOR_COUNT);
   }
   term_printf("\r\n");

} /* FlashEraseAll */

/*************************************************************************/
/*  FlashEraseArea                                                       */
/*                                                                       */
/*  Erase the given flash area.                                          */
/*                                                                       */
/*  In    : dStartAddress, dSize                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void FlashEraseArea (uint32_t dStartAddress, uint32_t dSize)
{
   uint32_t dBaseAddress = EPCS_FLASH_CONTROLLER_BASE + EPCS_FLASH_CONTROLLER_REGISTER_OFFSET;
   uint32_t dAddress;
   uint16_t wSectorCount;
   uint16_t wCount;
   
   wSectorCount = dSize / FLASH_SECTOR_SIZE;
   
   for (wCount = 0; wCount < wSectorCount; wCount++)
   {
      dAddress = dStartAddress + (wCount * FLASH_SECTOR_SIZE);
   
      epcs_write_enable(dBaseAddress);
      epcs_sector_erase(dBaseAddress, dAddress);
      term_printf(".");
   }

} /* FlashEraseArea */

/*************************************************************************/
/*  FlashProgramArea                                                     */
/*                                                                       */
/*  Program the given flash area.                                        */
/*                                                                       */
/*  In    : dStartAddress, pBuffer, dSize                                */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void FlashProgramArea (uint32_t dStartAddress, uint8_t *pBuffer, uint32_t dSize)
{
   uint32_t dBaseAddress = EPCS_FLASH_CONTROLLER_BASE + EPCS_FLASH_CONTROLLER_REGISTER_OFFSET;
   uint32_t dAddress;
   uint16_t wPageCount;
   uint16_t wCount;
   
   wPageCount = dSize / FLASH_PAGE_SIZE;
   
   dAddress = dStartAddress;  
   for (wCount = 0; wCount < wPageCount; wCount++)
   {
      epcs_write_buffer(dBaseAddress, dAddress, pBuffer, FLASH_PAGE_SIZE);
      dAddress += FLASH_PAGE_SIZE;
      pBuffer  += FLASH_PAGE_SIZE;
      
      if (0 == (wCount % 256))
      {
         term_printf(".");
      }   
   }

} /* FlashProgramArea */

/*** EOF ***/
